/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#include "saoriaddaccountdialog.h"
#include "saoriapplication.h"
#include "saoridef.h"
#include "ui_saoriaddaccountdialog.h"
#include <QDesktopServices>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QJsonDocument>
#include <QJsonObject>
#include <QEventLoop>
#include <QUrlQuery>
#include <QDebug>

SaoriAddAccountDialog::SaoriAddAccountDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SaoriAddAccountDialog)
{
    ui->setupUi(this);
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    for(int i = 0;i < SaoriApplication::getInstanceList()->count();i ++) {
        ui->instanceUrl_comboBox->addItem(SaoriApplication::getInstanceList()->at(i)->instance().toString());
    }
    ui->getAccessToken_pushButton->setEnabled(false);
    ui->authorizeCode_lineEdit->setEnabled(false);
    connect(ui->instanceUrl_comboBox,&QComboBox::editTextChanged,[this](QString) {
        m_instance = SaoriApplication::findInstance(QUrl(ui->instanceUrl_comboBox->currentText()));
        if (m_instance != nullptr) ui->openWebBrowser_pushButton->setEnabled(true);
        else ui->openWebBrowser_pushButton->setEnabled(false);
    });
}

SaoriAddAccountDialog::~SaoriAddAccountDialog()
{
    delete ui;
}

void SaoriAddAccountDialog::on_openWebBrowser_pushButton_clicked()
{
    m_instance = SaoriApplication::findInstance(ui->instanceUrl_comboBox->currentText());
    if (m_instance == nullptr) return;
    if (m_instance->clientId().isEmpty() || m_instance->clientSecret().isEmpty()) {
        qDebug() << "regist";
    }
    QDesktopServices::openUrl(m_instance->getAuthorizedUrl());
    QString text = ui->instanceUrl_comboBox->currentText();
    ui->instanceUrl_comboBox->clear();
    ui->instanceUrl_comboBox->addItem(text);
    ui->instanceUrl_comboBox->setEditable(false);
    ui->openWebBrowser_pushButton->setEnabled(false);
    ui->getAccessToken_pushButton->setEnabled(true);
    ui->authorizeCode_lineEdit->setEnabled(true);
    return;
}

void SaoriAddAccountDialog::on_getAccessToken_pushButton_clicked()
{
    if (ui->authorizeCode_lineEdit->text().isEmpty()) return;
    QString token = m_instance->getAccessToken(ui->authorizeCode_lineEdit->text());
    if (token.isEmpty()) return;
    m_account = new SaoriAccount("check",m_instance,token);

    QEventLoop event;
    QUrlQuery params;
    QNetworkRequest request;
    QString instance = m_instance->instanceInfo("uri");

    request.setRawHeader("Authorization","Bearer " + token.toLatin1());
    request.setUrl(QUrl(m_instance->instance().url() + SAORI_MASTODON_APIPATH_ACCOUNTS + "/verify_credentials"));
    auto reply = SaoriApplication::saori()->manager->get(request);
    connect(reply,&QNetworkReply::finished,&event,&QEventLoop::quit);
    event.exec();
    if (reply->error() != QNetworkReply::NoError) {
        reply->deleteLater();
        return;
    }
    QJsonObject json = QJsonDocument::fromJson(reply->readAll()).object();
    QString account = json["acct"].toString();
    reply->deleteLater();

    m_account->setName(account + "@" + instance);
    ui->account_lineEdit->setText(m_account->name());
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    return;

}

void SaoriAddAccountDialog::on_check_pushButton_clicked()
{
    if (SaoriApplication::findInstance(QUrl(ui->instanceUrl_comboBox->currentText())) == nullptr) {
        auto instance = new Saoridon(QUrl(ui->instanceUrl_comboBox->currentText()),this);
        connect(instance,&Saoridon::instanceInfomationChanged,this,[=](){
            if (!instance->instanceInfo("uri").isEmpty()) {
                instance->clientRedistration();
                if (!instance->clientSecret().isEmpty()) {
                    instance->setParent(SaoriApplication::saori());
                    SaoriApplication::getInstanceList()->append(instance);
                    ui->instanceUrl_comboBox->addItem(instance->instance().toString());
                    ui->instanceUrl_comboBox->setEditText(instance->instance().toString());
                    ui->openWebBrowser_pushButton->setEnabled(true);
                }
            }
        });
    }
}
